Package: BayesMallows
Type: Package
Title: Bayesian Preference Learning with the Mallows Rank Model
Version: 2.2.7
Authors@R: c(person("Oystein", "Sorensen",
                    email = "oystein.sorensen.1985@gmail.com",
                    role = c("aut", "cre"),
                    comment = c(ORCID = "0000-0003-0724-3542")),
            person("Waldir", "Leoncio",
                    email = "w.l.netto@medisin.uio.no",
                    role = c("aut")),
            person("Valeria", "Vitelli",
                    role = c("aut"),
                    email = "valeria.vitelli@medisin.uio.no",
                    comment = c(ORCID = "0000-0002-6746-0453")),
            person("Marta", "Crispino",
                    email = "crispino.marta8@gmail.com",
                    role = c("aut")),
            person("Qinghua", "Liu",
                    email = "qinghual@math.uio.no",
                    role = c("aut")),
            person("Cristina", "Mollica",
                    email = "cristina.mollica@uniroma1.it",
                    role = c("aut")),
            person("Luca", "Tardella",
                    role = c("aut")),
            person("Anja", "Stein",
                    role = c("aut"))            
                    )
Maintainer: Oystein Sorensen <oystein.sorensen.1985@gmail.com>
Description: An implementation of the Bayesian version of the Mallows rank model
    (Vitelli et al., Journal of Machine Learning Research, 2018 <https://jmlr.org/papers/v18/15-481.html>;
    Crispino et al., Annals of Applied Statistics, 2019 <doi:10.1214/18-AOAS1203>;
    Sorensen et al., R Journal, 2020 <doi:10.32614/RJ-2020-026>;
    Stein, PhD Thesis, 2023 <https://eprints.lancs.ac.uk/id/eprint/195759>). Both Metropolis-Hastings
    and sequential Monte Carlo algorithms for estimating the models are available. Cayley, footrule,
    Hamming, Kendall, Spearman, and Ulam distances are supported in the models. The rank data to be
    analyzed can be in the form of complete rankings, top-k rankings, partially missing rankings, as well
    as consistent and inconsistent pairwise preferences. Several functions for plotting and studying the
    posterior distributions of parameters are provided. The package also provides functions for estimating
    the partition function (normalizing constant) of the Mallows rank model, both with the importance
    sampling algorithm of Vitelli et al. and asymptotic approximation with the IPFP algorithm
    (Mukherjee, Annals of Statistics, 2016 <doi:10.1214/15-AOS1389>).
URL: https://github.com/ocbe-uio/BayesMallows,
        https://ocbe-uio.github.io/BayesMallows/
BugReports: https://github.com/ocbe-uio/BayesMallows/issues
License: GPL-3
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.3.3
Depends: R (>= 3.5.0)
Imports: Rcpp (>= 1.0.0), ggplot2 (>= 3.1.0), Rdpack (>= 1.0), sets (>=
        1.0-18), relations (>= 0.6-8), rlang (>= 0.3.1)
LinkingTo: Rcpp, RcppArmadillo, testthat
Suggests: knitr, testthat (>= 3.0.0), label.switching (>= 1.7),
        rmarkdown, covr, parallel (>= 3.5.1)
VignetteBuilder: knitr, rmarkdown
RdMacros: Rdpack
Config/testthat/edition: 3
NeedsCompilation: yes
Packaged: 2026-01-20 10:51:22 UTC; oyss
Author: Oystein Sorensen [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-0724-3542>),
  Waldir Leoncio [aut],
  Valeria Vitelli [aut] (ORCID: <https://orcid.org/0000-0002-6746-0453>),
  Marta Crispino [aut],
  Qinghua Liu [aut],
  Cristina Mollica [aut],
  Luca Tardella [aut],
  Anja Stein [aut]
Repository: CRAN
Date/Publication: 2026-01-20 11:20:08 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-26 11:44:22 UTC; unix
Archs: BayesMallows.so.dSYM
