# Generated by roxygen2: do not edit by hand

S3method("burnin<-",BayesMallows)
S3method("burnin<-",BayesMallowsMixtures)
S3method(assess_convergence,BayesMallows)
S3method(assess_convergence,BayesMallowsMixtures)
S3method(burnin,BayesMallows)
S3method(burnin,BayesMallowsMixtures)
S3method(burnin,SMCMallows)
S3method(compute_consensus,BayesMallows)
S3method(compute_consensus,SMCMallows)
S3method(compute_posterior_intervals,BayesMallows)
S3method(compute_posterior_intervals,SMCMallows)
S3method(generate_initial_ranking,BayesMallowsIntransitive)
S3method(generate_initial_ranking,BayesMallowsTransitiveClosure)
S3method(get_acceptance_ratios,BayesMallows)
S3method(get_acceptance_ratios,SMCMallows)
S3method(plot,BayesMallows)
S3method(plot,SMCMallows)
S3method(print,BayesMallows)
S3method(print,BayesMallowsMixtures)
S3method(print,SMCMallows)
S3method(update_mallows,BayesMallows)
S3method(update_mallows,BayesMallowsPriorSamples)
S3method(update_mallows,SMCMallows)
export("burnin<-")
export(assess_convergence)
export(assign_cluster)
export(burnin)
export(compute_consensus)
export(compute_exact_partition_function)
export(compute_expected_distance)
export(compute_mallows)
export(compute_mallows_mixtures)
export(compute_mallows_sequentially)
export(compute_observation_frequency)
export(compute_posterior_intervals)
export(compute_rank_distance)
export(create_ordering)
export(create_ranking)
export(estimate_partition_function)
export(get_acceptance_ratios)
export(get_cardinalities)
export(get_mallows_loglik)
export(get_transitive_closure)
export(heat_plot)
export(plot_elbow)
export(plot_top_k)
export(predict_top_k)
export(sample_mallows)
export(sample_prior)
export(set_compute_options)
export(set_initial_values)
export(set_model_options)
export(set_priors)
export(set_progress_report)
export(set_smc_options)
export(setup_rank_data)
export(update_mallows)
importFrom(Rcpp,sourceCpp)
importFrom(Rdpack,reprompt)
importFrom(rlang,.data)
importFrom(stats,aggregate)
importFrom(stats,density)
importFrom(utils,head)
useDynLib(BayesMallows, .registration = TRUE)
